"""
This module contains all colors and cmaps. They can be used directly when plotting, or in case of a cmap, we can use its string alias. The available cmap aliases are:

- `pf_grey`, `pf_white_grey`,
- `pf_blue`, `pf_white_blue`,
- `pf_red`, `pf_white_red`,
- `pf_blue_white_red`.

We can also use their reversed variant with sufix `_r`, such as `pf_grey_r`.

Examples
--------

```{python}
#| echo: false
import seaborn.objects as so
import seaborn as sns
import profiplots as pf

data = sns.load_dataset("titanic")
pf.set_theme(name="default")
```

::: {.panel-tabset}

#### Discrete

```{python}
(
    so.Plot(data=data, x="sex", y="survived")
    .add(so.Bar(), so.Agg(), color="sex", legend=False)
    .scale(color={"female": pf.color.RED, "male": pf.color.LIGHT_GREY})  # color mapping
    .label(title="Survival rate of titanic female passengers was significantly higher than male passengers")
)
```

#### Continuous

```{python}
(
    so.Plot(data=data, x="age", y="fare", color="age")
    .add(so.Dots(alpha=1))
    .scale(color="pf_blue")  # specification of cmap
)
```

:::
"""

import matplotlib.colors as _colors

BLUE = "#465A9B"
"""
Profinit blue color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.BLUE])
```

"""

RED = "#E63C41"
"""
Profinit red color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.RED])
```

"""

GREY = "#282828"
"""
Profinit grey color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.GREY])
```

"""

PURPLE = "#7D4191"
"""
Profinit purple color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.PURPLE])
```

"""

PINK = "#B5578D"
"""
Profinit pink color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.PINK])
```

"""

YELLOW = "#FFD21E"
"""
Profinit yellow color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.YELLOW])
```

"""

ORANGE = "#F3943B"
"""
Profinit orange color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.ORANGE])
```

"""

GREEN = "#41C34B"
"""
Profinit green color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.GREEN])
```

"""

AZURE = "#3DADE5"
"""
Profinit azure color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.AZURE])
```

"""

WHITE = "#FFFFFF"
"""
Profinit white color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.WHITE])
```

"""

BLACK = "#000000"
"""
Profinit black color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.BLACK])
```

"""


# LIGHT colors
LIGHT_GREY = "#AAAAAA"
"""
Profinit light grey color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.LIGHT_GREY])
```

"""


# DARK colors
DARK_BLUE = "#2B436C"
"""
Profinit dark blue color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.DARK_BLUE])
```

"""

DARK_GREEN = "#007938"
"""
Profinit dark green color from its theme.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette([pf.color.DARK_GREEN])
```

"""

# Multiple shades
BLUES = ["#ECEEF5", "#C7CDE1", "#A2ACCD", "#7D8BB8", BLUE]
"""
List of blue colors. From lightest to darkest.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette(pf.color.BLUES)
```

"""

GREYS = [
    "#D7D7D7",
    LIGHT_GREY,
    "#7D7D7D",
    "#555555",
    GREY
]
"""
List of grey colors. From lightest to darkest.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette(pf.color.GREYS)
```
"""
REDS = ["#F8EBEB", "#EBC3C2", "#DE9B99", "#D17270", RED]
"""
List of red colors. From lightest to darkest.

```{python}
#| echo: false
import seaborn as sns
import profiplots as pf

sns.color_palette(pf.color.REDS)
```
"""

# CMAPs
GREY_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_grey", colors=GREYS)
"""
Colors: Light grey - grey. Also available under "pf_grey".

```{python}
#| echo: false
import profiplots as pf

pf.color.GREY_CMAP
```
"""

WHITE_GREY_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_white_grey", colors=[WHITE, *GREYS])
"""
Colors: White - grey. Also available under "pf_white_grey".

```{python}
#| echo: false
import profiplots as pf

pf.color.WHITE_GREY_CMAP
```
"""

BLUE_RED_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_blue_red", colors=BLUES[::-1] + REDS)
"""
Colors: Blue - Red. Blue is for minimum values and Red is for maximum values. Also available under "pf_blue_red".

```{python}
#| echo: false
import profiplots as pf

pf.color.BLUE_RED_CMAP
```
"""


BLUE_WHITE_RED_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_blue_white_red", colors=BLUES[::-1] + [WHITE] + REDS)
"""
Colors: Blue - White - Red. Blue is for minimum values and Red is for maximum values. Use when both ends of the spectrum have strong semantic meaning. For example where minimum is negative correlation and maximum is positive correlation. Also available under "pf_blue_white_red".

```{python}
#| echo: false
import profiplots as pf

pf.color.BLUE_WHITE_RED_CMAP
```
"""

BLUE_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_blue", colors=BLUES)
"""
Colors: Blue. Light Blue is used for minimum values and darker Blue is for maximum values. Also available under "pf_blue".

```{python}
#| echo: false
import profiplots as pf

pf.color.BLUE_CMAP
```
"""

WHITE_BLUE_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_white_blue", colors=[WHITE, *BLUES])
"""
Colors: White - Blue. White is for minimum values and Blue is for maximum values. Also available under "pf_white_blue".

```{python}
#| echo: false
import profiplots as pf

pf.color.WHITE_BLUE_CMAP
```
"""

RED_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_red", colors=REDS)
"""
Colors: White - Red. Light Red is for minimum values and Red is for maximum values. Also available under "pf_red".

```{python}
#| echo: false
import profiplots as pf

pf.color.RED_CMAP
```
"""

WHITE_RED_CMAP = _colors.LinearSegmentedColormap.from_list(name="pf_white_red", colors=[WHITE, *REDS])
"""
Colors: White - Red. White is for minimum values and Red is for maximum values. Also available under "pf_white_red".

```{python}
#| echo: false
import profiplots as pf

pf.color.WHITE_RED_CMAP
```
"""



from profiplots import settings, theme
from profiplots.style import _utils


def colored() -> dict:
    """Changes color cycle to a colored type.

    ::: {.callout-important}

    This style is more-or-less for explorations. Finaliz visualizations should have all colors specified manually.

    :::

    Returns
    -------
    dict
        Dictionary containing the changed color cycle.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="sex", y="survived", color="embark_town")
        .theme(pf.style.colored())
        .add(so.Bar(alpha=1), so.Agg(), so.Dodge())
        .label(title="Survival of various groups of passengers")
    )
    ```

    """
    _utils.validate_active_style()

    return {"axes.prop_cycle": theme.COLOR_CYCLER[settings.active_theme], "image.cmap": "pf_blue_white_red"}


def grey() -> dict:
    """Changes color cycle to a grey type (no colors).

    Returns
    -------
    dict
        RC Dictionary containing the changed color cycle.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="sex", y="survived")
        .theme(pf.style.grey())
        .add(so.Bar(), so.Agg(), so.Dodge())
        .label(title="Survival of various groups of passengers")
    )
    ```
    """
    _utils.validate_active_style()

    return {"axes.prop_cycle": theme.GREY_CYCLER[settings.active_theme], "image.cmap": "pf_grey"}

def greyscale() -> dict:
    """Changes color cycle to a greyscale. Unlike `grey` style, it uses multiple shades of grey.

    Returns
    -------
    dict
        RC Dictionary containing the changed color cycle.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="sex", y="survived", color="embark_town")
        .theme(pf.style.greyscale())
        .add(so.Bar(alpha=1), so.Agg(), so.Dodge())
        .label(title="Survival of various groups of passengers")
    )
    ```
    """
    _utils.validate_active_style()

    return {"axes.prop_cycle": theme.GREYSCALE_CYCLER[settings.active_theme], "image.cmap": "pf_grey"}

def bluescale() -> dict:
    """Changes color cycle to a bluescale.

    Returns
    -------
    dict
        RC Dictionary containing the changed color cycle.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="sex", y="survived", color="embark_town")
        .theme(pf.style.bluescale())
        .add(so.Bar(alpha=1), so.Agg(), so.Dodge())
        .label(title="Survival of various groups of passengers")
    )
    ```
    """
    _utils.validate_active_style()

    return {"axes.prop_cycle": theme.BLUESCALE_CYCLER[settings.active_theme], "image.cmap": "pf_blue"}


def grid(x: bool | None = None, y: bool | None = None) -> dict:
    """Shows or hides grid in the image.

    Parameters
    ----------
    x : bool | None
        If True, then shows X grid. If False, hides it. If None, does nothing. Defaults to None.
    y : bool | None
        If True, then shows Y grid. If False, hides it. If None, does nothing. Defaults to None.

    Returns
    -------
    dict
        RC dictionary containing the new grid configuration.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="age", y="fare")
        .theme(pf.style.grid(x=True, y=True))
        .add(so.Dots())
        .label(title="Dependency between Age and Fare of Titanic passengers")
    )
    ```
    """
    _utils.validate_active_style()

    rc_dict = {}

    if x is True and y is True:
        rc_dict["axes.grid"] = True
        rc_dict["axes.grid.axis"] = "both"
    elif x is True:  # and y is False or None
        rc_dict["axes.grid"] = True
        rc_dict["axes.grid.axis"] = "x"
    elif y is True:  # and x is False or None
        rc_dict["axes.grid"] = True
        rc_dict["axes.grid.axis"] = "y"
    elif x is False and y is False:
        rc_dict["axes.grid"] = False
    else:
        # both are None => nothing changes
        pass

    return rc_dict


def ticks(x: bool | None = None, y: bool | None = None) -> dict:
    """Returns configuration of the plots that shows or hides ticks on x or y axis.

    Parameters
    ----------
    x : bool | None
        If True, then x axis ticks are shown. If False, they are hidden. If None, does nothing. Defaults to None.
    y : bool | None
        If True, then y axis ticks are shown. If False, they are hidden. If None, does nothing. Defaults to None.

    Returns
    -------
    dict
        Configuration of the plots that shows or hides ticks when applied.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="age", y="fare")
        .theme(pf.style.ticks(x=False, y=False))
        .add(so.Dots())
        .label(title="Dependency between Age and Fare.")
    )
    ```
    """
    _utils.validate_active_style()

    rc_dict = {}

    if x is not None:
        rc_dict.update({"xtick.top": False, "xtick.bottom": x, "xtick.labeltop": False, "xtick.labelbottom": x})
    if y is not None:
        rc_dict.update({"ytick.right": False, "ytick.left": y, "ytick.labelright": False, "ytick.labelleft": y})

    return rc_dict


def spines(
    left: bool | None = None, right: bool | None = None, top: bool | None = None, bottom: bool | None = None
) -> dict:
    """Adds or removes borders of the plots.

    Parameters
    ----------
    left : bool | None
        If True, then the left spine is added. If False, it is remoted. If None, nothing is altered. Defaults to None.
    right : bool | None
        If True, then the right spine is added. If False, it is remoted. If None, nothing is altered.. Defaults to None.
    top : bool | None
        If True, then the top spine is added. If False, it is remoted. If None, nothing is altered.. Defaults to None.
    bottom : bool | None
        If True, then the bottom spine is added. If False, it is remoted. If None, nothing is altered.. Defaults to None.

    Returns
    -------
    dict
        Configuration of the spines.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    (
        so.Plot(data=data, x="survived", y="sex")
        .theme(pf.style.spines(left=True, right=True, top=True, bottom=True))
        .add(so.Bar(), so.Agg(), so.Dodge())
        .label(title="Survival of various groups of passengers")
    )
    ```
    """
    _utils.validate_active_style()

    rc_dict = {}

    if left is not None:
        rc_dict["axes.spines.left"] = left
    if right is not None:
        rc_dict["axes.spines.right"] = right
    if top is not None:
        rc_dict["axes.spines.top"] = top
    if bottom is not None:
        rc_dict["axes.spines.bottom"] = bottom

    return rc_dict



from profiplots import settings


def validate_active_style():
    """ """
    if settings.active_theme is None:
        raise ValueError("Active style is set to None. Did you forget to call `set_theme`?")



"""
This package provides us with various styles. We can use them to modify the default theme look.
These styles include:

- `grey`: sets greyscale as the default color palette.
- `colored`: sets colored palette as the default.
- `grid`: adds grids to the plots.
- ...
"""

from profiplots.style._modifier import bluescale, colored, grey, greyscale, grid, spines, ticks

__all__ = [
    "grey",
    "greyscale",
    "bluescale",
    "colored",
    "grid",
    "spines",
    "ticks"
]



"""
This module contains implementation of profiles. Profiles is a combination of multiple settings made for a specific purpose. For example profile `exploration` was made to allow quick data exploration.
"""


from profiplots import settings as _settings
from profiplots import theme as _theme

__all__ = [
    "exploration",
    "publish",
    "presentation"
]


def exploration() -> dict:
    """Profile whose purpose is to be used for exploratory analysis.

    Returns
    -------
    dict
        Matplotlib rc settings.

    Examples
    -------

    ```{python}
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")

    (
        so.Plot(data=data, x="age", y="fare")
        .theme(pf.profile.exploration())
        .add(so.Dots())
        .label(title="Passenger's Age vs Fare")
    )
    ```

    """
    return _theme.EXPLORATION_PROFILE[_settings.active_theme]


def publish() -> dict:
    """Use this profile for final visualizations that we want to publish. It has a very clean look.

    Returns
    -------
    dict
        Matplotlib rc settings.

    Examples
    -------

    ```{python}
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")

    (
        so.Plot(data=data, x="age", y="fare")
        .theme(pf.profile.publish())
        .add(so.Dots())
        .label(title="Passenger's Age vs Fare")
    )
    ```

    """
    return _theme.PUBLISH_PROFILE[_settings.active_theme]

def presentation() -> dict:
    """Use this profile for presentation. The plot components are adjusted to be visible with a slide projector.

    Returns
    -------
    dict
        Matplotlib rc settings.

    Examples
    -------

    ```{python}
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")

    (
        so.Plot(data=data, x="age", y="fare")
        .theme(pf.profile.presentation())
        .add(so.Dots())
        .label(title="Passenger's Age vs Fare")
    )
    ```

    """
    return _theme.PRESENTATION_PROFILE[_settings.active_theme]



"""
Package contains implementation of various themes (for example `default`). It also contains their specific implementations mapped to theme names, such as `GREY_CYCLER`.
"""

from cycler import cycler as _cycler

from profiplots import settings as _settings

__all__ = ["GREY_CYCLER", "GREYSCALE_CYCLER", "COLOR_CYCLER", "PUBLISH_PROFILE", "EXPLORATION_PROFILE", "PRESENTATION_PROFILE"]

GREY_CYCLER = {"default": _cycler(color=["#7D7D7D", "#7D7D7D", "#7D7D7D", "#7D7D7D", "#7D7D7D", "#7D7D7D", "#7D7D7D"])}
"""Mapping of name of the theme to the color cycler of the theme with a grey color."""

GREYSCALE_CYCLER = {"default": _cycler(color=["#282828", "#555555", "#7D7D7D", "#AAAAAA", "#BEBEBE"])}
"""Mapping of name of the theme to the color cycler of the theme with greyscale colors (multiple shades of grey)."""

BLUESCALE_CYCLER = {"default": _cycler(color=["#465A9B", "#6B7BAF", "#909CC3", "#B5BDD7", "#C7CDE1"])}
"""Mapping of name of the theme to the color cycler of the theme with bluescale colors (multiple shades of blue)."""

COLOR_CYCLER = {"default": _cycler(color=["#465A9B", "#E63C41", "#B5578D", "#FFD21E", "#F3943B", "#41C34B", "#3DADE5"])}
"""Mapping of name of the theme to the color cycler of the theme with colors."""

PUBLISH_PROFILE = {
    "default": {
        # GENERAL
        "font.size": 10.0,
        # GRID
        "axes.grid": False,
        "axes.grid.axis": "both",
        "grid.linewidth": 0.5,
        "grid.color": "#D7D7D7",
        # TICKS
        "xtick.top": False,
        "xtick.bottom": True,
        "xtick.labeltop": False,
        "xtick.labelbottom": True,
        "xtick.color": "#D7D7D7",
        "xtick.major.width": 0.5,
        "ytick.right": False,
        "ytick.left": True,
        "ytick.labelright": False,
        "ytick.labelleft": True,
        "ytick.color": "#D7D7D7",
        "ytick.major.width": 0.5,
        # SPINES
        "axes.edgecolor": "#D7D7D7",
        "axes.linewidth": 0.8,
        "axes.spines.left": True,
        "axes.spines.right": False,
        "axes.spines.top": False,
        "axes.spines.bottom": True,
    }
}
"""Matplotlib RC settings for the publish profile."""

EXPLORATION_PROFILE = {
    "default": {
        # GENERAL
        "font.size": 10.0,
        # GRID
        "axes.grid": True,
        "axes.grid.axis": "both",
        "grid.linewidth": 0.5,
        "grid.color": "#D7D7D7",
        # TICKS
        "xtick.top": False,
        "xtick.bottom": True,
        "xtick.labeltop": False,
        "xtick.labelbottom": True,
        "xtick.color": "#D7D7D7",
        "xtick.major.width": 0.5,
        "ytick.right": False,
        "ytick.left": True,
        "ytick.labelright": False,
        "ytick.labelleft": True,
        "ytick.color": "#D7D7D7",
        "ytick.major.width": 0.5,
        # SPINES
        "axes.linewidth": 0.8,
        "axes.spines.left": False,
        "axes.spines.right": False,
        "axes.spines.top": False,
        "axes.spines.bottom": False,
    }
}
"""Matplotlib RC settings for the exploration profile."""

PRESENTATION_PROFILE = {
    "default": {
        # GENERAL
        "font.size": 12.0,
        # GRID
        "axes.grid": False,
        "axes.grid.axis": "both",
        "grid.linewidth": 1,
        "grid.color": "#BEBEBE",
        # TICKS
        "xtick.top": False,
        "xtick.bottom": True,
        "xtick.labeltop": False,
        "xtick.labelbottom": True,
        "xtick.color": "#BEBEBE",
        "xtick.major.width": 1,
        "ytick.right": False,
        "ytick.left": True,
        "ytick.labelright": False,
        "ytick.labelleft": True,
        "ytick.color": "#BEBEBE",
        "ytick.major.width": 1,
        # SPINES
        "axes.edgecolor": "#BEBEBE",
        "axes.linewidth": 1.25,
        "axes.spines.left": True,
        "axes.spines.right": False,
        "axes.spines.top": False,
        "axes.spines.bottom": True,
    }
}
"""Matplotlib RC settings for the presentation profile."""

assert all(k in _settings.SUPPORTED_THEMES for k in GREY_CYCLER), "Invalid theme name in `GREY_CYCLER`."
assert all(k in _settings.SUPPORTED_THEMES for k in COLOR_CYCLER), "Invalid theme name in `COLOR_CYCLER`."
assert all(k in _settings.SUPPORTED_THEMES for k in PUBLISH_PROFILE), "Invalid theme name in `PUBLISH_PROFILE`."
assert all(k in _settings.SUPPORTED_THEMES for k in EXPLORATION_PROFILE), "Invalid theme name in `EXPLORATION_PROFILE`."
assert all(k in _settings.SUPPORTED_THEMES for k in PRESENTATION_PROFILE), "Invalid theme name in `PRESENTATION_PROFILE`."



"""
Various settings that the application for its running.
"""

active_theme = None
"""Theme that was set by `set_theme` function."""

SUPPORTED_THEMES = [
    "default",
]
"""
Supported names of themes.
"""



"""
`profiplots` package enables us to use profinit plot styling in:

- `seaborn`,
- `matplotlib`.

The main function of this package is `set_theme`. It replaces default seaborn and matplotlib styling with our custom, Profinit styling. To try it out, just call:

```python
import profiplots as pf
pf.set_theme()
```

Now all `matplotlib` and `seaborn` plots will have our Profinit theme. These plots can be further customized. This is handled by submodules. Check out packages (and modules) `profiplots.style` and `profiplots.profile`. For even better control, we can use `profiplots.colors` to create perfectly colored plots.

"""

import matplotlib as _mpl
import matplotlib.pyplot as _plt
import seaborn.objects as _so

from profiplots import color, profile, settings, style, theme

__all__ = ["reset_theme", "set_theme", "set_style", "style_context", "style", "profile", "color", "settings", "theme"]

# register colormaps
_mpl.colormaps.register(color.WHITE_GREY_CMAP, force=True)
_mpl.colormaps.register(color.WHITE_GREY_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.GREY_CMAP, force=True)
_mpl.colormaps.register(color.GREY_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.WHITE_BLUE_CMAP, force=True)
_mpl.colormaps.register(color.WHITE_BLUE_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.BLUE_CMAP, force=True)
_mpl.colormaps.register(color.BLUE_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.WHITE_RED_CMAP, force=True)
_mpl.colormaps.register(color.WHITE_RED_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.RED_CMAP, force=True)
_mpl.colormaps.register(color.RED_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.BLUE_WHITE_RED_CMAP, force=True)
_mpl.colormaps.register(color.BLUE_WHITE_RED_CMAP.reversed(), force=True)
_mpl.colormaps.register(color.BLUE_RED_CMAP, force=True)
_mpl.colormaps.register(color.BLUE_RED_CMAP.reversed(), force=True)

_orig_rc_config = _mpl.rcParams.copy()
"""Original rc settings before calling the first `set_theme`."""

class _RCAesthetics:
    """ """

    def __init__(self, **kwargs):
        self.config = kwargs

    def __enter__(self):
        self._orig = _mpl.rcParams.copy()
        _mpl.rcParams.update(self.config)
        _so.Plot.config.theme.update(self.config)

    def __exit__(self, exc_type, exc_value, exc_tb):
        _so.Plot.config.theme.update(self._orig)
        _mpl.rcParams.update(self._orig)



def set_theme(name: str = "default"):
    """Calling this function will set `profiplots` styling as the default styling for `matplotlib` and `seaborn` plots.

    Their corresponding values can be found in `profiplots.color`.
    This function must be called before calling additional styling functions, like `colored`, `grid` etc.

    Parameters
    ----------
    name: str
        Name of the theme.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    data = sns.load_dataset("titanic")
    ```

    **No theme set**

    ```{python}
    (
        so.Plot(data=data, x="sex", y="survived")
        .add(so.Bar(), so.Agg())
        .label(title="Survival rate of titanic female passengers was significantly higher than male passengers")
    )
    ```

    **Profiplots theme**

    ```{python}
    import profiplots as pf

    # set theme
    pf.set_theme(name="default")

    (
        so.Plot(data=data, x="sex", y="survived")
        .add(so.Bar(), so.Agg())
        .label(title="Survival rate of titanic female passengers was significantly higher than male passengers")
    )
    ```
    """
    if name not in settings.SUPPORTED_THEMES:
        raise ValueError(f"Theme with name '{name}' is not supported.")

    # reset theme before setting up the new one
    reset_theme()

    # set up global themes
    _plt.style.use(f"profiplots.theme.{name}")
    _so.Plot.config.theme.update(_mpl.rcParams)

    settings.active_theme = name


def reset_theme():
    """Reset theme to previous defaults."""
    _mpl.rcParams.update(_orig_rc_config)
    _so.Plot.config.theme.update(_orig_rc_config)
    settings.active_theme = None


def style_context(rc_config: dict):
    """Alters default Profinit theme with new settings in a context window. Works both for `matplotlib` and `seaborn` (and `seaborn.objects.Plot`).

    Use this method instead of [mpl.rc_context](https://matplotlib.org/stable/api/matplotlib_configuration_api.html#matplotlib.rc_context).

    Parameters
    ----------
    rc_config : dict
        rc config values to be set for the duration of the context window.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    First let's look at what happens **inside** of the style context.

    ```{python}
    # Inside style context
    with pf.style_context(pf.style.grid(x=True, y=True)):
        p = (
            so.Plot(data=data, x="age", y="fare")
            .add(so.Dots())
            .label(title="Title example")
            .plot()
        )
    p
    ```

    And now **outside** the default styles are being used..

    ```{python}
    # Inside style context
    p = (
        so.Plot(data=data, x="age", y="fare")
        .add(so.Dots())
        .label(title="Title example")
        .plot()
    )
    p
    ```
    """
    return _RCAesthetics(**rc_config)


def set_style(rc_config: dict):
    """Sets up the specific style permanently.

    Parameters
    ----------
    rc_config : dict
        rc config values to be set.

    Examples
    -------

    ```{python}
    #| echo: false
    import seaborn.objects as so
    import seaborn as sns
    import profiplots as pf

    data = sns.load_dataset("titanic")
    pf.set_theme(name="default")
    ```

    ```{python}
    # set styles
    pf.set_style(pf.style.colored())

    (
        so.Plot(data=data, x="age", y="fare", color="sex")
        .add(so.Dots())
    )
    ```
    """
    # set up global themes
    _plt.rcParams.update(**rc_config)
    _so.Plot.config.theme.update(_mpl.rcParams)
